﻿using System;
using System.Data.SqlClient;
using System.Diagnostics;
using System.Windows.Forms;
using PerpetuumSoft.Reporting.Components;
using PerpetuumSoft.Reporting.View;

namespace DataSetPerformance
{
    public partial class MainForm : Form
    {
        private readonly Stopwatch _stopwatch = new Stopwatch();

        public MainForm()
        {
            InitializeComponent();

            LoadData();
            dataTableReportSlot.RenderCompleted += PreviewReport;
            xmlReportSlot.RenderCompleted += PreviewReport;
        }

        /// <summary>
        /// Retrieves data for our report. 
        /// 
        /// Data for all projects in this solution is autogenerated 
        /// by http://www.fakenamegenerator.com and stored as a script 
        /// in GenerateTestData.sql - you will need to execute the script
        /// before running the application.
        /// </summary>
        private void LoadData()
        {
            var connection = new SqlConnection("Data Source=.; User Id=sa; Password=sa; Initial Catalog=PerformanceTest");
            var adapter = new SqlDataAdapter("SELECT * FROM fakenames", connection);
            adapter.Fill(usersDataTable);

            //the code below exports the table to xml - which can be later used by xml-based report
            //usersDataTable.WriteXml("fakenames.xml");
        }

        /// <summary>
        /// Displays the prepared report, also showing the time measurement. 
        /// 
        /// Due to asyncronous nature of Sharp-Shooter, 
        /// a report gets generated on a separate thread
        /// and this handler gets called when the generation
        /// is done. Here we don't always need to show the report: 
        /// we could have saved it, exported it, etc.
        /// </summary>
        /// <param name="sender">Report slot that has been just rendered.</param>
        /// <param name="e">Not used, is here because of C# coding convention.</param>
        private void PreviewReport(object sender, EventArgs e)
        {
            NotifyElapsedTime();

            using (var form = new PreviewForm((IReportSource)sender)) {
                form.WindowState = FormWindowState.Maximized;
                form.ShowDialog(this);
            }
        }

        /// <summary>
        /// Starts time measurement and generates the DataTable based report. 
        /// 
        /// Here we only start the process of rendering: once generation is done,
        /// our PreviewReport handler gets called.
        /// </summary>
        /// <param name="sender">Button that was clicked.</param>
        /// <param name="e">Info about mouse click, in our case not needed.</param>
        private void btnPreviewDataTableReport_Click(object sender, EventArgs e)
        {
            _stopwatch.Start();
            dataTableReportSlot.Prepare();
        }

        /// <summary>
        /// Starts time measurement and generates the XML based report. 
        /// 
        /// Here we only start the process of rendering: once generation is done,
        /// our PreviewReport handler gets called.
        /// </summary>
        /// <param name="sender">Button that was clicked.</param>
        /// <param name="e">Info about mouse click, in our case not needed.</param>
        private void btnPreviewXmlReport_Click(object sender, EventArgs e)
        {
            _stopwatch.Start();
            xmlReportSlot.Prepare();
        }

        private void NotifyElapsedTime()
        {
            _stopwatch.Stop();
            MessageBox.Show("Time elapsed: " + _stopwatch.Elapsed);
            _stopwatch.Reset();
        }
    }
}
